//
// Copyright (c) 2012, MindFusion LLC - Bulgaria.
//

using System;
using System.Data;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

using MindFusion.Drawing;

using SolidBrush = MindFusion.Drawing.SolidBrush;


namespace MindFusion.Diagramming.WinForms.Samples.CS.Telephony
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			diagramView.ShowToolTips = true;
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
			lvTypes.Items[0].Selected = true;
			diagram.Bounds = new RectangleF(0, 0, 2000, 3000);	// pixels
			diagramView.ScrollTo(0, 0);

			if (File.Exists(Application.StartupPath + "\\Diagram.tel"))
				diagram.LoadFromFile(Application.StartupPath + "\\Diagram.tel");

			// show instructions box
			ShapeNode ib = new ShapeNode(diagram);
			ib.Bounds = new RectangleF(10, 10, 250, 50);
			ib.Text = "To create a node, select node type from the list on the right, then " +
				"single-click in the drawing surface on the left.";
			ib.Tag = 42;
			ib.Brush = new MindFusion.Drawing.LinearGradientBrush(
				Color.White, Color.Gold, 30);
			ib.Shape = Shapes.Rectangle;
			diagram.Nodes.Add(ib);
			ib.ZTop(false);
		}

		private void CreateNode(int type, PointF location)
		{
			ShapeNode atcm = null;
			TableNode tbl = null;

			RectangleF bounds = new RectangleF(location, new SizeF(34, 48));
			bounds.Offset(-16, -16);

			ShapeNode node = new ShapeNode(diagram);
			node.Bounds = bounds;
			node.Transparent = true;
			node.EnabledHandles = AdjustmentHandles.Move; // no resizing
			node.Tag = type;
			diagram.Nodes.Add(node);

			switch (type)
			{
				case 0:	// start
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[6];
						node.Text = "Start";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
						break;
					}
				case 1:	// stop
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[7];
						node.Text = "Stop";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
						break;
					}
				case 2:	// play
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[1];
						node.Text = "Play";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;

						atcm = new ShapeNode(diagram);
						atcm.Bounds = new RectangleF(node.Bounds.Right, node.Bounds.Top + 10, 80, 32);
						atcm.Locked = true;
						atcm.Brush = new SolidBrush(Color.DodgerBlue);
						atcm.Text = "Choose a sound file...";
						atcm.Shape = Shapes.RoundRect;
						atcm.AttachTo(node, AttachToNode.TopLeft);
						diagram.Nodes.Add(atcm);
						break;
					}
				case 3:	// input sequence
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[8];
						node.Text = "Input";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;

						tbl = diagram.Factory.CreateTableNode(
							node.Bounds.Right + 2, node.Bounds.Top,
							80, 49, 2, 3);

						tbl.RowHeight = 16;
						tbl.Columns[0].Width = 50;
						tbl.EnabledHandles = AdjustmentHandles.Move;
						tbl.HandlesStyle = HandlesStyle.Invisible;
						tbl.Brush = new SolidBrush(Color.FromArgb(0, Color.White));
						tbl.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(20, Color.Black), 0);
						tbl.CellFrameStyle = CellFrameStyle.Simple;
						tbl.Caption = "";
						tbl.CaptionHeight = 0;
						tbl[0, 0].Text = "length";
						tbl[0, 1].Text = "timeout";
						tbl[0, 2].Text = "cancel";
						tbl.AttachTo(node, AttachToNode.TopLeft);

						break;
					}
				case 4: // input digit
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[8];
						node.Text = "Input";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
					}
					break;
				case 5:
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[10];
						node.Text = "Logic";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
					}
					break;
				case 6:
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[0];
						node.Text = "Rec";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
					}
					break;
				case 7: // email
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[4];
						node.Text = "EMail";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
					}
					break;
				case 8: // fax
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[5];
						node.Text = "Fax";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
					}
					break;
				case 9: // database
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[9];
						node.Text = "DB";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
					}
					break;
				case 10: // security code
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[3];
						node.Text = "Code";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
					}
					break;
				case 11: // timeout
					{
						node.ImageAlign = ImageAlign.TopLeft;
						node.Image = ilIcons.Images[2];
						node.Text = "Timeout";
						node.TextFormat.Alignment = StringAlignment.Center;
						node.TextFormat.LineAlignment = StringAlignment.Far;
					}
					break;
			}
		}

		private void PromptText(DiagramLink a)
		{
			TextForm f = new TextForm();
			f.TheText = a.Text;
			f.Title = "Enter Link Text";

			if (f.ShowDialog(this) == DialogResult.OK)
				a.Text = f.TheText;
		}

		private void menuItem2_Click(object sender, System.EventArgs e)
		{
			diagram.ClearAll();
		}

		private void menuItem3_Click(object sender, System.EventArgs e)
		{
			ofd.InitialDirectory = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			if (sfd.ShowDialog(this) == DialogResult.OK)
				diagram.SaveToFile(sfd.FileName, true);
		}

		private void menuItem4_Click(object sender, System.EventArgs e)
		{
			ofd.InitialDirectory = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			if (ofd.ShowDialog(this) == DialogResult.OK)
			{
				try
				{
					diagram.LoadFromFile(ofd.FileName);
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.Message);
				}
			}
		}

		private void menuItem6_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void contextMenu1_Popup(object sender, System.EventArgs e)
		{
			clickPoint = diagramView.ClientToDoc(diagramView.PointToClient(MousePosition));
		}

		private void menuItem7_Click(object sender, System.EventArgs e)
		{
			ShapeNode b = new ShapeNode(diagram);
			b.Bounds = new RectangleF(clickPoint.X, clickPoint.Y, 120, 60);
			b.Brush = new MindFusion.Drawing.LinearGradientBrush(
				Color.LightYellow, Color.Moccasin, 90);
			b.Font = new Font("Tahoma", 7.5f);
			b.HandlesStyle = HandlesStyle.Invisible;
			b.Shape = Shapes.Rectangle;
			b.PolygonalTextLayout = true;
			b.TextFormat.Alignment = StringAlignment.Center;
			b.Tag = -1;

			TextForm f = new TextForm();
			f.Title = "Enter Note Text";

			if (f.ShowDialog(this) == DialogResult.OK)
				b.Text = f.TheText;

			diagram.Nodes.Add(b);
		}

		private void diagram_NodeTextEditing(object sender, MindFusion.Diagramming.NodeValidationEventArgs e)
		{
			e.Cancel = true;
		}

		private void diagram_LinkCreated(object sender, MindFusion.Diagramming.LinkEventArgs e)
		{
			PromptText(e.Link);
		}

		private void diagram_Clicked(object sender, MindFusion.Diagramming.DiagramEventArgs e)
		{
			if (e.MouseButton == MouseButton.Left)
			{
				if (lvTypes.SelectedItems.Count == 0)
					return;

				CreateNode(lvTypes.SelectedItems[0].Index, e.MousePosition);

				DiagramNode ib = diagram.FindNode(42);
				if (ib != null)
					diagram.Nodes.Remove(ib);
			}
			else
			{
				diagram.Selection.Clear();
				contextMenu1.Show(this, diagramView.DocToClient(e.MousePosition));
			}
		}

		private void diagram_NodeDoubleClicked(object sender, MindFusion.Diagramming.NodeEventArgs e)
		{
			ShapeNode node = e.Node as ShapeNode;
			if (node == null)
				return;

			if ((int)e.Node.Tag == 7)
			{
				TextForm f = new TextForm();
				f.TheText = e.Node.ToolTip;
				f.Title = "Enter Recepient";
				if (f.ShowDialog(this) == DialogResult.OK)
					e.Node.ToolTip = f.TheText;
			}
			else
				if ((int)e.Node.Tag == -1) // note
				{
					TextForm f = new TextForm();
					f.TheText = node.Text;
					f.Title = "Enter Note Text";
					if (f.ShowDialog(this) == DialogResult.OK)
						node.Text = f.TheText;
				}
		}

		private void diagram_NodeClicked(object sender, MindFusion.Diagramming.NodeEventArgs e)
		{
			ShapeNode node = e.Node as ShapeNode;
			if (node == null)
				return;

			if (node.MasterGroup != null)
			{
				if (ofd.ShowDialog() == DialogResult.OK)
					node.Text = System.IO.Path.GetFileName(ofd.FileName);
			}
		}

		// GroupDestroyed means that items are not longer grouped, and not that 
		// all items are deleted. That usually happens if the group's master item is
		// deleted, and the event handler is a good place to delete the subordinates
		private void diagram_GroupDestroyed(object sender, MindFusion.Diagramming.GroupEventArgs e)
		{
			while (e.Group.AttachedNodes.Count > 0)
				diagram.Nodes.Remove(e.Group.AttachedNodes[0]);
		}

		private void diagram_CellTextEditing(object sender, MindFusion.Diagramming.CellValidationEventArgs e)
		{
			if (e.Column == 0)
				e.Cancel = true;
		}


		private PointF clickPoint;
	}
}
